@@ -1,6 +1,8 @@
 #include <qpdf/Pl_LZWDecoder.hh>
 
 #include <qpdf/QTC.hh>
+#include <qpdf/QUtil.hh>
+#include <qpdf/QIntC.hh>
 #include <stdexcept>
 #include <string.h>
 #include <assert.h>
@@ -51,22 +53,22 @@ Pl_LZWDecoder::finish()
 void
 Pl_LZWDecoder::sendNextCode()
 {
-    int high = this->byte_pos;
-    int med = (this->byte_pos + 1) % 3;
-    int low = (this->byte_pos + 2) % 3;
+    unsigned int high = this->byte_pos;
+    unsigned int med = (this->byte_pos + 1) % 3;
+    unsigned int low = (this->byte_pos + 2) % 3;
 
-    int bits_from_high = 8 - this->bit_pos;
-    int bits_from_med = this->code_size - bits_from_high;
-    int bits_from_low = 0;
+    unsigned int bits_from_high = 8 - this->bit_pos;
+    unsigned int bits_from_med = this->code_size - bits_from_high;
+    unsigned int bits_from_low = 0;
     if (bits_from_med > 8)
     {
 	bits_from_low = bits_from_med - 8;
 	bits_from_med = 8;
     }
-    int high_mask = (1 << bits_from_high) - 1;
-    int med_mask = 0xff - ((1 << (8 - bits_from_med)) - 1);
-    int low_mask = 0xff - ((1 << (8 - bits_from_low)) - 1);
-    int code = 0;
+    unsigned int high_mask = (1U << bits_from_high) - 1U;
+    unsigned int med_mask = 0xff - ((1U << (8 - bits_from_med)) - 1U);
+    unsigned int low_mask = 0xff - ((1U << (8 - bits_from_low)) - 1U);
+    unsigned int code = 0;
     code += (this->buf[high] & high_mask) << bits_from_med;
     code += ((this->buf[med] & med_mask) >> (8 - bits_from_med));
     if (bits_from_low)
@@ -93,20 +95,29 @@ Pl_LZWDecoder::sendNextCode()
 }
 
 unsigned char
-Pl_LZWDecoder::getFirstChar(int code)
+Pl_LZWDecoder::getFirstChar(unsigned int code)
 {
     unsigned char result = '\0';
     if (code < 256)
     {
 	result = static_cast<unsigned char>(code);
     }
+    else if (code > 257)
+    {
+	unsigned int idx = code - 258;
+	if (idx >= table.size())
+        {
+            throw std::runtime_error(
+                "Pl_LZWDecoder::getFirstChar: table overflow");
+        }
+	Buffer& b = table.at(idx);
+	result = b.getBuffer()[0];
+    }
     else
     {
-	assert(code > 257);
-	unsigned int idx = code - 258;
-	assert(idx < table.size());
-	Buffer& b = table[idx];
-	result = b.getBuffer()[0];
+        throw std::runtime_error(
+            "Pl_LZWDecoder::getFirstChar called with invalid code (" +
+            QUtil::int_to_string(code) + ")");
     }
     return result;
 }
@@ -120,18 +131,27 @@ Pl_LZWDecoder::addToTable(unsigned char next)
 
     if (this->last_code < 256)
     {
-	tmp[0] = this->last_code;
+	tmp[0] = static_cast<unsigned char>(this->last_code);
 	last_data = tmp;
 	last_size = 1;
     }
+    else if (this->last_code > 257)
+    {
+	unsigned int idx = this->last_code - 258;
+	if (idx >= table.size())
+        {
+            throw std::runtime_error(
+                "Pl_LZWDecoder::addToTable: table overflow");
+        }
+	Buffer& b = table.at(idx);
+	last_data = b.getBuffer();
+	last_size = QIntC::to_uint(b.getSize());
+    }
     else
     {
-	assert(this->last_code > 257);
-	unsigned int idx = this->last_code - 258;
-	assert(idx < table.size());
-	Buffer& b = table[idx];
-	last_data = b.getBuffer();
-	last_size = b.getSize();
+        throw std::runtime_error(
+            "Pl_LZWDecoder::addToTable called with invalid code (" +
+            QUtil::int_to_string(this->last_code) + ")");
     }
 
     Buffer entry(1 + last_size);
@@ -142,7 +162,7 @@ Pl_LZWDecoder::addToTable(unsigned char next)
 }
 
 void
-Pl_LZWDecoder::handleCode(int code)
+Pl_LZWDecoder::handleCode(unsigned int code)
 {
     if (this->eod)
     {
@@ -170,11 +190,11 @@ Pl_LZWDecoder::handleCode(int code)
 	    // be what we read last plus the first character of what
 	    // we're reading now.
 	    unsigned char next = '\0';
-	    unsigned int table_size = table.size();
+	    unsigned int table_size = QIntC::to_uint(table.size());
 	    if (code < 256)
 	    {
 		// just read < 256; last time's next was code
-		next = code;
+		next = static_cast<unsigned char>(code);
 	    }
 	    else if (code > 257)
 	    {
@@ -219,7 +239,13 @@ Pl_LZWDecoder::handleCode(int code)
 	}
 	else
 	{
-	    Buffer& b = table[code - 258];
+            unsigned int idx = code - 258;
+            if (idx >= table.size())
+            {
+                throw std::runtime_error(
+                    "Pl_LZWDecoder::handleCode: table overflow");
+            }
+	    Buffer& b = table.at(idx);
 	    getNext()->write(b.getBuffer(), b.getSize());
 	}
     }
